/*                                                                          
        Source: cl_status.c
        Author: Adam G. Bowen (agbowen@bealenet.com)
        $Revision$
        $Date$

 * Program: Command Line status viewer for NetSaint
 * License: GPL
 * Copyright (c) 1999 Adam G. Bowen (agbowen@bealenet.com)
 *
 * Description:
 *
 * This program will display the current host and service status totals and the
 * last 10 services to be checked by netsaint.
 *
 * Command line:
 *
 * cl_status [-c <config file>]
 *
 * Required input:
 *
 * none
 *
 * Optional input:
 *
 * [-c <config file>] = Alternate configuration file to use.
 *
 * Note:
 *
 * If [-c <config file>] is not passed on the command line, then the deafult
 * configuration file will be used.
 *
 * $Log$
*/

#include "command_line.h"

int main(int argc, char *argv[])
{
  char command_name[MAX_CHARS];
  char config_file[MAX_CHARS];
  char count_refresh[MAX_CHARS];
  char status_file[MAX_CHARS];
  char temp_config_file[MAX_CHARS];
  char temp_file[MAX_CHARS];

  int bad_argument;
  int delay_count;
  int key_press;
  int loop_count;
  int refresh_count;
  int result;
  int return_value;
  int use_default_config;
  int wait_for_key_press;

  /* Initialize init signal handling */

  signal(SIGINT,exit_program);
  signal(SIGQUIT,exit_program);
  signal(SIGTERM,exit_program);

  /* Initialize variables */

  /* Initialize variables */

  bad_argument = FALSE;
  use_default_config = TRUE;

  /* Get command name from command line */

  strcpy(command_name,get_command_name(argv[0]));

  /* Check for command line arguments and set variables */

  if(argc > 1)
  {
    for(loop_count = 1;loop_count < argc;loop_count++)
    {
      if((strcmp(argv[loop_count],"-c")) == 0)
      {
        if (loop_count >= (argc -1))
        {
          bad_argument = TRUE;
        } 
        else
        {
          loop_count++;
          strcpy(temp_config_file,argv[loop_count]); 
          use_default_config=FALSE;
        }
      }
      else
      {
        bad_argument = TRUE;
      }
    }
  }

  if(bad_argument == TRUE)
  {
    printf(" Incorrect arguments supplied\n");
    printf("\n");
    printf(" Command line status viewer for NetSaint\n");
    printf(" Copyright (c) 1999 Adam G. Bowen (agbowen@bealenet.com)\n");
    printf(" $Revision$\n");
    printf(" Last Modified $Date$\n");
    printf(" License: GPL\n");
    printf("\n");
    printf(" Description:\n");
    printf("\n");
    printf(" This program will display the current host and service status totals and the\n");
    printf(" last 10 services to be checked by netsaint.\n");
    printf("\n");
    printf(" Usage: %s [-c <config file>]\n",command_name);
    printf("\n");
    printf(" Required input:\n");
    printf("\n");
    printf(" none\n");
    printf("\n");
    printf(" Optional input:\n");
    printf("\n");
    printf(" [-c <config file>] = Alternate configuration file to use.\n");
    printf("\n");
    printf(" Note:\n");
    printf("\n");
    printf(" If [-c <config file>] is not passed on the command line, then the deafult\n");
    printf(" %s configuration file will be used.\n",DEFAULT_CONFIG_FILE);
    printf("\n");
    return_value = STATE_UNKNOWN;
  }
  else
  {
    if(use_default_config == TRUE)
    {
      strcpy(config_file, DEFAULT_CONFIG_FILE);
    }
    else
    {
      strcpy(config_file, temp_config_file);
    }
    if (access(config_file, EXISTS) != 0 )
    {
      printf("Config file %s does not exist!\n",config_file);
      return_value = STATE_UNKNOWN;
    }
    else
    {
      if((return_value=get_var("STATUS_FILE", config_file, status_file)) != STATE_OK)
      {
        printf("STATUS_FILE entry not found in config file %s!\n", config_file);
      }
      else if((return_value=get_var("REFRESH_TIME", config_file, count_refresh)) != STATE_OK)
      {
        printf("REFRESH_TIME entry not found in config file %s!\n", config_file);
      }
      else
      {
        refresh_count=atoi(count_refresh);
        delay_count=refresh_count * 10;
        initscr();
        noecho();
        nonl();
        intrflush(stdscr, FALSE);
        keypad(stdscr, TRUE);

        erase();
        refresh();

        show_static();

        do
        {
          strcpy(temp_file, tmpnam(NULL));
          hoststatus_list=NULL;
          servicestatus_list=NULL;
        
          show_update_time();

          /* sort the status.log into temp_file */
          result=sort_status_log(status_file, temp_file);
          if(result != OK)
          {
            printf("Error in sorting status log\n");
            exit(1);
          }
          /* get status information */
          result=get_status_info(temp_file);
          if(result != OK)
          {
            printf("Error in getting status info\n");
            exit(2);
          }

          remove(temp_file);

          /* show status totals */
          show_status_totals();

          /* show service details */
          show_service_detail();

          free_host_status_memory();
          free_service_status_memory();

          refresh();
          wait_for_key_press=halfdelay(delay_count);
          key_press=getch();
        }
        while(key_press != 113);
      }
    }
  }
  endwin();
  return return_value;
}
