/*
        Source: command_line.h 
        Author: Adam G. Bowen (agbowen@bealenet.com) 
        $Revision$
        $Date$


 * Header file for command line status viewer for Netsaint program.
 *
 * $Log$
 *
*/

#include <ctype.h>
#include <curses.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

/* service state return codes */

#define START_DETAIL_DISPLAY_LINE 7
#define MAX_LINES 25

#define STATE_CRITICAL 2
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

//#define TRUE           1
//#define FALSE          0
#define EXISTS         4

#define MAX_CHARS      256

#define DEFAULT_CONFIG_FILE     "/usr/local/netsaint/etc/cl_status.cfg"

//#define        OK                              0
#define ERROR                          -2

#define SERVICE_PENDING                 1
#define SERVICE_OK                      2
#define SERVICE_WARNING                 4
#define SERVICE_UNKNOWN                 8
#define SERVICE_CRITICAL                16
#define SERVICE_RECOVERY                32
#define SERVICE_HOST_DOWN               64
#define SERVICE_UNREACHABLE             128

#define HOST_PENDING                    1
#define HOST_UP                         2
#define HOST_DOWN                       4
#define HOST_UNREACHABLE                8

#define SOFT_STATE                      0
#define HARD_STATE                      1

/* HOST STATUS structure */
typedef struct hoststatus_struct
{
  char     host_name[MAX_CHARS];
  char     information[MAX_CHARS];
  int      status;
  time_t   last_check;
  time_t   last_state_change;
  unsigned long   time_up;
  unsigned long   time_down;
  unsigned long   time_unreachable;
  time_t   last_notification;
  int      notifications_enabled;
  struct   hoststatus_struct *next;
}hoststatus;

/* SERVICE STATUS structure */
typedef struct servicestatus_struct
{
  char     host_name[MAX_CHARS];
  char     description[MAX_CHARS];
  char     information[MAX_CHARS];
  int      max_attempts;
  int      current_attempt;
  int      status;
  time_t   last_check;
  time_t   next_check;
  int      enabled;
  time_t   last_state_change;
  int      last_hard_state;
  int      state_type;
  unsigned long	time_ok;
  unsigned long	time_unknown;
  unsigned long	time_warning;
  unsigned long	time_critical;
  time_t   last_notification;
  int      notifications_enabled;
  struct   servicestatus_struct *next;
}servicestatus;

hoststatus    *hoststatus_list=NULL;
servicestatus *servicestatus_list=NULL;

int get_var(char *LINE_NAME, char *file_name, char *var_name)
{
  char temp_input_buffer[MAX_CHARS];
  char tok_sep[] = "|";
  char temp_var_name[MAX_CHARS];
  char *temp_var_value;

  FILE *temp_fp;

  int return_value = STATE_UNKNOWN;

  strcpy(var_name,"NOT_FOUND");

  temp_fp=fopen(file_name,"r");
  while(fgets(temp_input_buffer,MAX_CHARS-1,temp_fp) != NULL)
  {
    strcpy(temp_var_name,strtok(temp_input_buffer,tok_sep));
    temp_var_value=strtok(NULL,tok_sep);
    if(strcmp(LINE_NAME,temp_var_name) == 0)
    {
      strcpy(var_name,temp_var_value);
      return_value = STATE_OK;
    }
  }
  return return_value;
}
char *get_command_name(char *command_line)
{
  char *return_string;
  char *sub_string;
  if((sub_string=strrchr(command_line,'/')) != NULL)
  {
    return_string=++sub_string;
  }
  else
  {
    return_string=command_line;
  }
  return return_string;
}
/* display service status totals */
void show_status_totals(void)
{
  char total_critical_string[MAX_CHARS];
  char total_down_string[MAX_CHARS];
  char total_ok_string[MAX_CHARS];
  char total_pend_host_string[MAX_CHARS];
  char total_pend_serv_string[MAX_CHARS];
  char total_unreachable_string[MAX_CHARS];
  char total_unknown_string[MAX_CHARS];
  char total_up_string[MAX_CHARS];
  char total_warning_string[MAX_CHARS];

  hoststatus *temp_hoststatus;

  int result;
  int total_critical=0;
  int total_down=0;
  int total_ok=0;
  int total_pending_host=0;
  int total_pending_service=0;
  int total_unreachable=0;
  int total_unknown=0;
  int total_up=0;
  int total_warning=0;

  servicestatus *temp_servicestatus;

  move(2,0);
  clrtoeol();
  move(3,0);
  clrtoeol();

  /* check the status of all services... */
  for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next)
  {
    if(temp_servicestatus->status==SERVICE_CRITICAL)
    {
      total_critical++;
    }
    else if(temp_servicestatus->status==SERVICE_HOST_DOWN)
    {
      total_critical++;
    }
    else if(temp_servicestatus->status==SERVICE_UNREACHABLE)
    {
      total_critical++;
    }
    else if(temp_servicestatus->status==SERVICE_WARNING)
    {
      total_warning++;
    }
    else if(temp_servicestatus->status==SERVICE_UNKNOWN)
    {
      total_unknown++;
    }
    else if(temp_servicestatus->status==SERVICE_RECOVERY)
    {
      total_ok++;
    }
    else if(temp_servicestatus->status==SERVICE_OK)
    {
      total_ok++;
    }
    else if(temp_servicestatus->status==SERVICE_PENDING)
    {
      total_pending_service++;
    }
    else
    {
      total_ok++;
    }
  }
  for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next)
  {
    if(temp_hoststatus->status==HOST_UP)
    {
      total_up++;
    }
    else if(temp_hoststatus->status==HOST_DOWN)
    {
      total_down++;
    }
    else if(temp_hoststatus->status==HOST_UNREACHABLE)
    {
      total_unreachable++;
    }
    else if(temp_hoststatus->status==HOST_PENDING)
    {
      total_pending_host++;
    }
    else
    {
      total_up++;
    }
  }
/*

 The following code will make the console beep and flash if there are any
 errors.  If curses cannot make the console flash, it will beep again.  
 Comment out this section and rerun make if you do not want the beeps 

*/
  if((total_critical > 0) || (total_down > 0) || (total_ok == 0) || (total_unreachable > 0) || (total_unknown > 0) || (total_up = 0) || (total_warning > 0))
  {
    result=beep();
    result=flash();
  }
/*

 End beep and flash

*/

  sprintf(total_critical_string,"%d", total_critical);
  sprintf(total_down_string,"%d", total_down);
  sprintf(total_ok_string,"%d", total_ok);
  sprintf(total_pend_host_string,"%d", total_pending_host);
  sprintf(total_pend_serv_string,"%d", total_pending_service);
  sprintf(total_unreachable_string,"%d", total_unreachable);
  sprintf(total_unknown_string,"%d", total_unknown);
  sprintf(total_up_string,"%d", total_up);
  sprintf(total_warning_string,"%d", total_warning);

  if(total_pending_host >0)
  {
    mvaddstr(2, 1, "Up");
    mvaddstr(2, 4, "Down");
    mvaddstr(2, 9, "Unreachable");
    mvaddstr(2, 21, "Pending");
  }
  else
  {
    mvaddstr(2, 4, "Up");
    mvaddstr(2, 8, "Down");
    mvaddstr(2, 13, "Unreachable");
  }

  if(total_unknown >0)
  {
    if(total_pending_service >0)
    {
      mvaddstr(2, 43, "Ok");
      mvaddstr(2, 47, "Warning");
      mvaddstr(2, 55, "Critical");
      mvaddstr(2, 64, "Unknown");
      mvaddstr(2, 72, "Pending");
    }
    else
    {
      mvaddstr(2, 48, "Ok");
      mvaddstr(2, 52, "Warning");
      mvaddstr(2, 60, "Critical");
      mvaddstr(2, 69, "Unknown");
    }
  }
  else if(total_pending_service > 0)
  {
    mvaddstr(2, 48, "Ok");
    mvaddstr(2, 52, "Warning");
    mvaddstr(2, 60, "Critical");
    mvaddstr(2, 69, "Pending");
  }
  else
  {
    mvaddstr(2, 51, "Ok");
    mvaddstr(2, 55, "Warning");
    mvaddstr(2, 63, "Critical");
  }

  if(total_pending_host >0)
  {
    mvaddstr(3, 1, total_up_string);
    mvaddstr(3, 5, total_down_string);
    mvaddstr(3, 14, total_unreachable_string);
    mvaddstr(3, 24, total_pend_host_string);
  }
  else
  {
    mvaddstr(3, 4, total_up_string);
    mvaddstr(3, 9, total_down_string);
    mvaddstr(3, 18, total_unreachable_string);
  }

  if(total_unknown >0)
  {
    if(total_pending_service >0)
    {
      mvaddstr(3, 43, total_ok_string);
      mvaddstr(3, 50, total_warning_string);
      mvaddstr(3, 58, total_critical_string);
      mvaddstr(3, 67, total_unknown_string);
      mvaddstr(3, 75, total_pend_serv_string);
    }
    else
    {
      mvaddstr(3, 48, total_ok_string);
      mvaddstr(3, 54, total_warning_string);
      mvaddstr(3, 63, total_critical_string);
      mvaddstr(3, 72, total_unknown_string);
    }
  }
  else if(total_pending_service > 0)
  {
    mvaddstr(3, 48, total_ok_string);
    mvaddstr(3, 54, total_warning_string);
    mvaddstr(3, 63, total_critical_string);
    mvaddstr(3, 72, total_pend_serv_string);
  }
  else
  {
    mvaddstr(3, 51, total_ok_string);
    mvaddstr(3, 58, total_warning_string);
    mvaddstr(3, 66, total_critical_string);
  }

  return;
}
/* display a detailed listing of the status of the last services */
void show_service_detail(void)
{
  char attempt[MAX_CHARS];
  char date_time[MAX_CHARS];
  char host_name[MAX_CHARS];
  char service_info[MAX_CHARS];
  char service_name[MAX_CHARS];
  char status[MAX_CHARS];

  int line_count;

  servicestatus *temp_status;

  struct tm *time_ptr;

  time_t t;

  line_count=START_DETAIL_DISPLAY_LINE;
  move(line_count,0);
  clrtobot();

  for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next)
  {
    /* get the last service check time */
    t=temp_status->last_check;
    time_ptr=localtime(&t);
    strftime(date_time,MAX_CHARS,"%b %d %H:%M:%S\n",time_ptr);
    if(temp_status->last_check==0L)
    {
      strcpy(date_time,"N/A");
    }

    if(temp_status->status==SERVICE_PENDING)
    {
      strncpy(status,"PENDING",sizeof(status));
    }
    else if(temp_status->status==SERVICE_OK)
    {
      strncpy(status,"OK",sizeof(status));
    }
    else if(temp_status->status==SERVICE_RECOVERY)
    {
      strncpy(status,"RECOVERY",sizeof(status));
    }
    else if(temp_status->status==SERVICE_WARNING){
      strncpy(status,"WARNING",sizeof(status));
    }
    else if(temp_status->status==SERVICE_UNKNOWN)
    {
      strncpy(status,"UNKNOWN",sizeof(status));
    }
    else if(temp_status->status==SERVICE_CRITICAL)
    {
      strncpy(status,"CRITICAL",sizeof(status));
    }
    else if(temp_status->status==SERVICE_HOST_DOWN)
    {
      strncpy(status,"HOST DOWN",sizeof(status));
    }
    else if(temp_status->status==SERVICE_UNREACHABLE)
    {
      strncpy(status,"UNREACHABLE",sizeof(status));
    }

    strncpy(host_name,temp_status->host_name,11);
    strncpy(service_name,temp_status->description,8);
    strncpy(service_info,temp_status->information,32);
    status[sizeof(status)-1]='\x0';

    sprintf(attempt, "%d/%d\t",temp_status->current_attempt,temp_status->max_attempts);

    mvaddstr(line_count, 0, "           ");
    mvaddstr(line_count, 0, host_name);
    mvaddstr(line_count, 11, " ");
    mvaddstr(line_count, 12, "        ");
    mvaddstr(line_count, 12, service_name);
    mvaddstr(line_count, 20, " ");
    mvaddstr(line_count, 21, "      ");
    mvaddstr(line_count, 21, status);
    mvaddstr(line_count, 27, " ");
    mvaddstr(line_count, 28, "               ");
    mvaddstr(line_count, 28, date_time);
    mvaddstr(line_count, 43, " ");
    mvaddstr(line_count, 44, "   ");
    mvaddstr(line_count, 44, attempt);
    mvaddstr(line_count, 47, " ");
    mvaddstr(line_count, 48, "                                ");
    mvaddstr(line_count, 48, service_info);

    line_count++;
    if(line_count > MAX_LINES)
    {
      break;
    }
  }
  return;
}
/* read the contents of the log file into memory*/
int get_status_info(char *filename)
{
  char input_buffer[MAX_CHARS];
  char status[MAX_CHARS];
  char *temp_buffer;

  hoststatus *last_hoststatus;
  hoststatus *new_hoststatus;
  hoststatus *temp_hoststatus;

  servicestatus *new_svcstatus;

  FILE *fp;

  fp=fopen(filename,"r");
  if(fp==NULL)
  return ERROR;

  while((fgets(input_buffer,MAX_CHARS-1,fp)) != NULL)
  {

    if(feof(fp))
    {
      break;
    }

    if(input_buffer==NULL)
    {
      continue;
    }

    /* service status information... */
    else if(strstr(input_buffer,"] SERVICE;"))
    {

      /* allocate memory for a new service status entry */
      new_svcstatus=(servicestatus *)malloc(sizeof(servicestatus));
      if(new_svcstatus!=NULL)
      {

        temp_buffer=strtok(input_buffer,"]");
        new_svcstatus->last_check=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
        temp_buffer=strtok(NULL,";");
        temp_buffer=strtok(NULL,";");
        strncpy(new_svcstatus->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_CHARS);
        new_svcstatus->host_name[MAX_CHARS-1]='\x0';
        temp_buffer=strtok(NULL,";");
        strncpy(new_svcstatus->description,(temp_buffer==NULL)?"":temp_buffer,MAX_CHARS);
        new_svcstatus->description[MAX_CHARS-1]='\x0';
        temp_buffer=strtok(NULL,";");
        strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
        status[sizeof(status)-1]='\x0';
        if(!strcmp(status,"WARNING"))
          new_svcstatus->status=SERVICE_WARNING;
        else if(!strcmp(status,"UNKNOWN"))
          new_svcstatus->status=SERVICE_UNKNOWN;
        else if(!strcmp(status,"CRITICAL"))
          new_svcstatus->status=SERVICE_CRITICAL;
        else if(!strcmp(status,"RECOVERY"))
          new_svcstatus->status=SERVICE_RECOVERY;
        else if(!strcmp(status,"HOST DOWN"))
          new_svcstatus->status=SERVICE_HOST_DOWN;
        else if(!strcmp(status,"UNREACHABLE"))
          new_svcstatus->status=SERVICE_UNREACHABLE;
        else if(!strcmp(status,"PENDING"))
          new_svcstatus->status=SERVICE_PENDING;
        else
          new_svcstatus->status=SERVICE_OK;
        temp_buffer=strtok(NULL,"/");
        new_svcstatus->current_attempt=(temp_buffer==NULL)?0:atoi(temp_buffer);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->state_type=(!strcmp((temp_buffer==NULL)?"":temp_buffer,"SOFT"))?SOFT_STATE:HARD_STATE;
        temp_buffer=strtok(NULL,";");
        new_svcstatus->next_check=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->last_state_change=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
        status[sizeof(status)-1]='\x0';
        if(!strcmp(status,"WARNING"))
          new_svcstatus->last_hard_state=SERVICE_WARNING;
        else if(!strcmp(status,"UNKNOWN"))
          new_svcstatus->last_hard_state=SERVICE_UNKNOWN;
        else if(!strcmp(status,"CRITICAL"))
          new_svcstatus->last_hard_state=SERVICE_CRITICAL;
        else
          new_svcstatus->last_hard_state=SERVICE_OK;
        temp_buffer=strtok(NULL,";");
        new_svcstatus->time_ok=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->time_unknown=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->time_warning=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->time_critical=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->last_notification=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_svcstatus->notifications_enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
        temp_buffer=strtok(NULL,"\n");
        strncpy(new_svcstatus->information,(temp_buffer==NULL)?"":temp_buffer,MAX_CHARS);
        new_svcstatus->information[MAX_CHARS-1]='\x0';
  

        /* add new service status to front of list */
        if(servicestatus_list==NULL)
        {
          new_svcstatus->next=NULL;
          servicestatus_list=new_svcstatus;
        }
        else
        {
          new_svcstatus->next=servicestatus_list;
          servicestatus_list=new_svcstatus;
        }
      }
    }
    /* host status information... */
    else if(strstr(input_buffer,"] HOST;"))
    {
      /* allocate memory for new host status */
      new_hoststatus=(hoststatus *)malloc(sizeof(hoststatus));
      if(new_hoststatus!=NULL)
      {
        temp_buffer=strtok(input_buffer,"]");
        new_hoststatus->last_check=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
        temp_buffer=strtok(NULL,";");
        temp_buffer=strtok(NULL,";");
        strncpy(new_hoststatus->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_CHARS);
        new_hoststatus->host_name[MAX_CHARS-1]='\x0';
        temp_buffer=strtok(NULL,";");
        strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
        status[sizeof(status)-1]='\x0';
        temp_buffer=strtok(NULL,";");
        new_hoststatus->last_state_change=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_hoststatus->time_up=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_hoststatus->time_down=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_hoststatus->time_unreachable=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_hoststatus->last_notification=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
        temp_buffer=strtok(NULL,";");
        new_hoststatus->notifications_enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
        temp_buffer=strtok(NULL,"\n");
        strncpy(new_hoststatus->information,(temp_buffer==NULL)?"":temp_buffer,MAX_CHARS);
        new_hoststatus->information[MAX_CHARS-1]='\x0';

        if(!strcmp(status,"DOWN"))
        {
          new_hoststatus->status=HOST_DOWN;
        }
        else if(!strcmp(status,"UNREACHABLE"))
        {
          new_hoststatus->status=HOST_UNREACHABLE;
        }
        else if(!strcmp(status,"PENDING"))
        {
          new_hoststatus->status=HOST_PENDING;
        }
        else
        {
          new_hoststatus->status=HOST_UP;
        }

        /* add new host status to list, sorted by host name */
        last_hoststatus=hoststatus_list;
        for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next)
        {
          if(strcmp(new_hoststatus->host_name,temp_hoststatus->host_name)<0)
          {
            new_hoststatus->next=temp_hoststatus;
            if(temp_hoststatus==hoststatus_list)
            {
              hoststatus_list=new_hoststatus;
            }
            else
            {
              last_hoststatus->next=new_hoststatus;
            }
            break;
          }
          else
          {
            last_hoststatus=temp_hoststatus;
          }
        }
        if(hoststatus_list==NULL)
        {
          new_hoststatus->next=NULL;
          hoststatus_list=new_hoststatus;
        }
        else if(temp_hoststatus==NULL)
        {
          new_hoststatus->next=NULL;
          last_hoststatus->next=new_hoststatus;
        }
      }
    }
  }
  fclose(fp);
  return OK;
}
int sort_status_log(char *status_file, char *temp_file)
{
  char command_line[MAX_CHARS];
  
  sprintf(command_line,"cat %s|sort > %s",status_file, temp_file);
  system(command_line);
  return OK;
}
/* free memory allocated to the status list */
void free_service_status_memory(void)
{
  servicestatus *this_svcstatus;
  servicestatus *next_svcstatus;

  /* free memory for the service status list */
  this_svcstatus=servicestatus_list;
  while(this_svcstatus!=NULL)
  {
    next_svcstatus=this_svcstatus->next;
    free(this_svcstatus);
    this_svcstatus=next_svcstatus;
  }
return;
}
/* free memory allocated to the host status list */
void free_host_status_memory(void)
{
  hoststatus *this_hoststatus;
  hoststatus *next_hoststatus;

  /* free memory for the host status list */
  this_hoststatus=hoststatus_list;
  while(this_hoststatus!=NULL)
  {
    next_hoststatus=this_hoststatus->next;
    free(this_hoststatus);
    this_hoststatus=next_hoststatus;
  }

  return;
}
void exit_program(int sig_num)
{
  endwin();
  printf("\nThank you for using the command line status display.\n");
  exit(STATE_OK);
}
void show_static(void)
{
  mvaddstr(0, 0, "Netsaint Network Monitoring System");
  mvaddstr(0, 50, "Last Updated:");
  mvaddstr(1, 5, "Host Status Totals");
  mvaddstr(1, 51, "Service Status Totals");
  mvaddstr(4, 16, "Most Resent Service Status Details For All Hosts");
  mvaddstr(5, 4, "Host");
  mvaddstr(5, 12, "Service");
  mvaddstr(5, 21, "Status");
  mvaddstr(5, 30, "Last Update");
  mvaddstr(5, 42, "Attempt");
  mvaddstr(5, 55, "Service Information");
  mvaddstr(6, 0, "--------------------------------------------------------------------------------");
}
void show_update_time(void)
{
  char date_time[48];

  struct tm *time_ptr;

  time_t current_time;

  time(&current_time);
  time_ptr=localtime(&current_time);
  strftime(date_time,MAX_CHARS,"%b %d %H:%M:%S\n",time_ptr);
  move(0, 64);
  clrtoeol();
  mvaddstr(0, 64, date_time);
}
